#ifndef CGUL_INT_H
#define CGUL_INT_H

/**
 * Exact integer types. <p>
 *
 * @file
 * @author Paul Serice
 * @brief exact integer types
 */

#include "cgul_common.h"

CGUL_BEGIN_C

#if (CHAR_BIT != 8)
#    error "cgul_int assumes bytes are octets"
#endif

/**
 * 8-bit signed integer.
 */
typedef signed char cgul_int8_t;

/**
 * 8-bit unsigned integer.
 */
typedef unsigned char cgul_uint8_t;

/**
 * 16-bit signed integer.
 */
typedef short int cgul_int16_t;

/**
 * 16-bit unsigned integer.
 */
typedef unsigned short int cgul_uint16_t;

/**
 * 32-bit signed integer.
 */
#if (INT_MAX == 2147483647)
    typedef int cgul_int32_t;
#elif (LONG_MAX == 2147483647L)
    typedef long int cgul_int32_t;
#else
    #error "Unable to determine type for cgul_int32_t"
#endif

/**
 * 32-bit unsigned integer.
 */
#if (UINT_MAX == 4294967295U)
    typedef unsigned int cgul_uint32_t;
#elif (ULONG_MAX == 4294967295UL)
    typedef unsigned long int cgul_uint32_t;
#else
    #error "Unable to determine type for cgul_int32_t"
#endif

/**
 * Macro to test whether the system supports 64-bit signed integers.
 * If the system supports 64-bit signed integers, the macro will be
 * defined.
 */
#undef CGUL_INT64

/* Some compilers cannot handle the large constants below so prevent
 * them from ever reaching that code. */
#if (LONG_MAX > 2147483647L)
/**
 * 64-bit signed integer.
 */
#if (LONG_MAX == 9223372036854775807L)
    typedef long int cgul_int64_t;
#   define CGUL_INT64 1
#elif (defined(LLONG_MAX) && (LLONG_MAX == 9223372036854775807LL))
    typedef long long int cgul_int64_t;
#   define CGUL_INT64 1
#endif
#endif

/**
 * Macro to test whether the system supports 64-bit unsigned integers.
 * If the system supports 64-bit unsigned integers, the macro will be
 * defined.
 */
#undef CGUL_UINT64

/* Some compilers cannot handle the large constants below so prevent
 * them from ever reaching that code. */
#if (ULONG_MAX > 4294967295UL)
/**
 * 64-bit unsigned integer.
 */
#if (ULONG_MAX == 18446744073709551615UL)
    typedef unsigned long int cgul_uint64_t;
#   define CGUL_UINT64 1
#elif (defined(ULLONG_MAX) && (ULLONG_MAX == 18446744073709551615ULL))
    typedef unsigned long long int cgul_uint64_t;
#   define CGUL_UINT64 1
#endif
#endif

CGUL_END_C

#endif  /*  CGUL_INT_H  */
