#ifndef CGUL_BLOCK_READER_P_H
#define CGUL_BLOCK_READER_P_H

#include "cgul_block_reader.h"

CGUL_BEGIN_C

/**
 * Virtual destructor.  Derived classes should override this method to
 * deallocate resources as necessary.  Classes that implement this
 * interface must correctly chain their destructor with their parent
 * class's destructor.
 */
typedef void
(*cgul_block_reader__virtual_delete_t)(
    cgul_block_reader_t br);

/**
 * Virtual method that reads one block of data <code>block</code> of
 * size <code>block_size</code> bytes from an external data source.
 * The user is responsible for making sure <code>block</code> points to
 * at least <code>block_size</code> contiguous bytes of memory.  This
 * method returns the number of bytes read or <code>0</code> if EOF is
 * reached without error.  If an error occurs, <code>0</code> is
 * returned, and an exception is thrown. <p>
 *
 * @param[in,out] cex c-style exception
 * @param[in] br <code>cgul_block_reader</code> instance
 * @param[in] block
 * @param[in] block_size block size in bytes
 * @return number of bytes read
 */
typedef size_t
(*cgul_block_reader__virtual_read_next_block_t)(
    cgul_exception_t* cex,
    cgul_block_reader_t br,
    void* block,
    size_t block_size);

/**
 * Virtual method that returns the presentation name.
 *
 * @param[in,out] cex c-style exception
 * @param[in] br <code>cgul_block_reader</code> instance
 * @return presentation name
 */
typedef const char*
(*cgul_block_reader__virtual_get_presentation_name_t)(
    cgul_exception_t* cex,
    cgul_block_reader_t br);

/**
 * Virtual method that sets the presentation name.  The presentation
 * name is primarily used for error reporting and can be set to
 * anything, but it does need to be set again if the input source
 * changes.  An exception is thrown if an attempt is made to set the
 * presentation name without first setting the input source.
 *
 * @param[in,out] cex c-style exception
 * @param[in] br <code>cgul_block_reader</code> instance
 * @param[in] presentation_name presentation name
 */
typedef void
(*cgul_block_reader__virtual_set_presentation_name_t)(
    cgul_exception_t* cex,
    cgul_block_reader_t br,
    const char* presentation_name);

/**
 * Virtual function table (vtable) for the
 * <code>cgul_block_reader</code> interface.  Derived classes are
 * responsible for allocating and initializing the (typically static)
 * vtable.
 */
typedef struct cgul_block_reader__vtable {

    /**
     * Virtual destructor.
     */
    cgul_block_reader__virtual_delete_t
    virtual_delete;

    /**
     * Virtual method for reading blocks.
     */
    cgul_block_reader__virtual_read_next_block_t
    virtual_fread;

    /**
     * Virtual method for getting the presentation name.
     */
    cgul_block_reader__virtual_get_presentation_name_t
    virtual_get_presentation_name;

    /**
     * Virtual method for setting the presentation name.
     */
    cgul_block_reader__virtual_set_presentation_name_t
    virtual_set_presentation_name;

} *cgul_block_reader__vtable_t;

/**
 * Data encapsulated by the <code>cgul_block_reader</code> interface.
 * This structure is essentially a virtual function table.
 */
struct cgul_block_reader {

    /**
     * Virtual Function Table Pointer (vptr).
     *
     * Derived classes are responsible for initializing this structure
     * to point to their (typically static) virtual function table.
     */
    cgul_block_reader__vtable_t vptr;

};

CGUL_END_C

#endif  /*  CGUL_BLOCK_READER_P_H  */
