#include "cgul_crc16_incr_p.h"
#include "cgul_libc.h"

cgul_crc16_incr_t
cgul_crc16_incr__new(cgul_exception_t* cex,
                     unsigned short g,
                     unsigned short init,
                     unsigned short final_xor,
                     int reflected)
{
    cgul_crc16_incr_t rv = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    /* Allocate the object. */
    rv = (cgul_crc16_incr_t)cgul_libc__calloc(cex, 1, sizeof(*rv));
    if (*cex) {
        goto out;
    }

    /* Allocate the components. */
    rv->incr_crc =
        cgul_crc16__new(cex, g, init, final_xor, reflected);
    rv->final_crc =
        cgul_crc16__new(cex, g, init, final_xor, reflected);
    if (*cex) {
        goto out;
    }

 out:

    /* Clean up. */
    if (*cex && rv) {
        cgul_crc16_incr__delete(rv);
        rv = NULL;
    }

    return rv;
}

cgul_crc16_incr_t
cgul_crc16_incr__new_appends_size(cgul_exception_t* cex,
                                  unsigned short g,
                                  unsigned short init,
                                  unsigned short final_xor,
                                  int reflected,
                                  size_t size_xor)
{
    cgul_crc16_incr_t rv = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    /* Allocate the object. */
    rv = (cgul_crc16_incr_t)cgul_libc__calloc(cex, 1, sizeof(*rv));
    if (*cex) {
        goto out;
    }

    /* Allocate the components. */
    rv->incr_crc =
        cgul_crc16__new_appends_size(cex,
                                     g,
                                     init,
                                     final_xor,
                                     reflected,
                                     size_xor);
    rv->final_crc =
        cgul_crc16__new_appends_size(cex,
                                     g,
                                     init,
                                     final_xor,
                                     reflected,
                                     size_xor);
    if (*cex) {
        goto out;
    }

 out:

    /* Clean up. */
    if (*cex && rv) {
        cgul_crc16_incr__delete(rv);
        rv = NULL;
    }

    return rv;
}

void
cgul_crc16_incr__delete(cgul_crc16_incr_t crc16)
{
    if (crc16) {

        /* final_crc */
        if (crc16->final_crc) {
            cgul_crc16__delete(crc16->final_crc);
        }

        /* incr_crc */
        if (crc16->incr_crc) {
            cgul_crc16__delete(crc16->incr_crc);
        }

        /* Free the object itself. */
        free(crc16);
    }
}

void
cgul_crc16_incr__assign(cgul_exception_t* cex,
                        cgul_crc16_incr_t lhs,
                        cgul_crc16_incr_t rhs)
{
    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(lhs);
    assert(rhs);

    if (lhs == rhs) {
        goto out;
    }

    cgul_crc16__assign(cex, lhs->incr_crc, rhs->incr_crc);
    cgul_crc16__assign(cex, lhs->final_crc, rhs->final_crc);
    if (*cex) {
        goto out;
    }

 out:
    return;
}

unsigned long
cgul_crc16_incr__crc(cgul_exception_t* cex,
                     cgul_crc16_incr_t crc)
{
    unsigned long rv = 0;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(crc);

    /* Assign the incremental state to final_crc. */
    cgul_crc16__assign(cex, crc->final_crc, crc->incr_crc);
    if (*cex) {
        goto out;
    }

    /* Return the final crc. */
    rv = cgul_crc16__crc(cex, crc->final_crc);

 out:
    return rv;
}

void
cgul_crc16_incr__update_block(cgul_exception_t* cex,
                              cgul_crc16_incr_t crc,
                              const void* block,
                              size_t length)
{
    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(crc);
    assert(block);

    cgul_crc16__update_block(cex, crc->incr_crc, block, length);

 out:
    return;
}

void
cgul_crc16_incr__update_file(cgul_exception_t* cex,
                             cgul_crc16_incr_t crc,
                             FILE* f)
{
    size_t rcount = 0;
    size_t block_size = 16384;
    unsigned char* block = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(crc);
    assert(f);

    /* Allocate. */
    block = (unsigned char*)cgul_libc__calloc(cex, 1, block_size);
    if (*cex) {
        goto out;
    }

    /* Iterate over each block in the file. */
    for (;;) {

        /* Read the next block. */
        rcount = cgul_libc__fread(cex, block, 1, block_size, f);
        if (*cex) {
            goto out;
        }

        /* Check for EOF. */
        if (rcount == 0) {
            goto out;
        }

        /* Update the CRC with this block. */
        cgul_crc16_incr__update_block(cex, crc, block, rcount);
    }

 out:

    /* Clean up. */
    if (block) {
        free(block);
    }

    return;
}

void
cgul_crc16_incr__update_fname(cgul_exception_t* cex,
                              cgul_crc16_incr_t crc,
                              const char* fname)
{
    FILE* f = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(crc);
    assert(fname);

    /* Open the file. */
    f = cgul_libc__fopen(cex, fname, "rb");
    if (*cex) {
        goto out;
    }

    /* Update the CRC with the file. */
    cgul_crc16_incr__update_file(cex, crc, f);
    if (*cex) {
        goto out;
    }

 out:

    /* Clean up. */
    if (f) {
        fclose(f);
    }

    return;
}

void
cgul_crc16_incr__reset(cgul_exception_t* cex,
                       cgul_crc16_incr_t crc)
{
    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(crc);

    cgul_crc16__reset(cex, crc->incr_crc);
    cgul_crc16__reset(cex, crc->final_crc);

 out:
    return;
}
