#include "cgul_log2.h"

int
cgul_log2__floor(cgul_exception_t* cex,
                 unsigned long int n)
{
    int rv = 0;

    assert(cex);
    if (*cex) {
        goto out;
    }

    /* Sanity check. */
    if (n <= 0) {
        *cex = cgul_exception__new("cgul_log2__floor: log of zero");
        goto out;
    }

    /* Count the significant bits in n. */
    while (n >>= 1) {
        /* Each right shift of n is another power of 2. */
        ++rv;
    }

 out:
    return rv;
}

int
cgul_log2__ceil(cgul_exception_t* cex,
                unsigned long int n)
{
    int rv = 0;

    assert(cex);
    if (*cex) {
        goto out;
    }

    /* Sanity check. */
    if (n <= 0) {
        *cex = cgul_exception__new("cgul_log2__ceil: log of zero");
        goto out;
    }

    /* Determine the floor. */
    rv = cgul_log2__floor(cex, n);
    if (*cex) {
        goto out;
    }

    /* Round up if necessary. */
    if (n > (1UL << rv)) {
        ++rv;
    }

 out:
    return rv;
}
