#include "cgul_trie_node_with_indexing_p.h"
#include "cgul_libc.h"

/*
 * This method is used as the "restructor" for the "cgul_cache" of
 * nodes in the "cgul_trie".  "restructor" methods are defined in
 * "cgul_cache" to not throw exceptions.
 *
 * While the signature for this method does not exactly match the
 * signature defined for "restructor" methods, it can be safely
 * typecast to match.
 */
void
cgul_trie_node_with_indexing__initialize(cgul_trie_node_with_indexing_t n)
{
    assert(n);
    memset(n, 0, sizeof(*n));
}

cgul_trie_node_with_indexing_t
cgul_trie_node_with_indexing__new(cgul_exception_t* cex)
{
    cgul_trie_node_with_indexing_t rv = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    rv = (cgul_trie_node_with_indexing_t)cgul_libc__malloc(cex, sizeof(*rv));
    if (*cex) {
        goto out;
    }

    cgul_trie_node_with_indexing__initialize(rv);

 out:

    /* Delete partially constructed objects. */
    if (*cex && rv) {
        cgul_trie_node_with_indexing__delete(rv);
        rv = NULL;
    }

    return rv;
}

void
cgul_trie_node_with_indexing__delete(cgul_trie_node_with_indexing_t n)
{
    if (n) {
        free(n);
    }
}
