#include <assert.h>
#include "cgul.h"

typedef struct {
    int x;
} *myobj_t;

/* Restructor */
static void
myobj__initialize(myobj_t obj)
{
    if (obj) {
        obj->x = 3;
    }
}

/* Destructor */
static void
myobj__delete(myobj_t obj)
{
    if (obj) {
        free(obj);
    }
}

/* Constructor */
static myobj_t
myobj__new(cgul_exception_t* cex)
{
    myobj_t rv = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    rv = cgul_libc__calloc(cex, 1, sizeof(*rv));
    if (*cex) {
        goto out;
    }

    myobj__initialize(rv);

 out:

    if (*cex && rv) {
        myobj__delete(rv);
        rv = NULL;
    }

    return rv;
}

int main(int argc, char* argv[])
{
    int rv = 0;
    unsigned long int i = 0;
    unsigned long int reserve = 0;
    myobj_t obj = NULL;
    cgul_exception_t local = NULL;
    cgul_cache_t cache = NULL;

    /* Create a new cache object. */
    cache = cgul_cache__new(&local,
                             (cgul_cache__constructor_t)myobj__new,
                             (cgul_cache__destructor_t)myobj__delete,
                             (cgul_cache__restructor_t)myobj__initialize);
    if (local) {
        goto out;
    }

    /* Reserve 3 cache objects.  This instantiates all 3 objects and
     * reserves them (until the reserve level is reduced). */
    reserve = 3;
    cgul_cache__set_reserve(&local, cache, reserve);

    /* Make the reserved objects available. */
    cgul_cache__set_reserve(&local, cache, 0);

    /* Get each reserved object. */
    for (i = 0 ; i < reserve ; ++i) {
        obj = cgul_cache__get(&local, cache);
        cgul_libc__printf(&local, "obj = %p\n", obj);
        cgul_cache__put(&local, cache, obj);
    }

 out:

    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    if (cache) {
        cgul_cache__delete(cache);
    }

    return rv;
}
