#include "cgul.h"

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    int i = 0;
    unsigned long value = 0;
    cgul_string_t label = NULL;
    cgul_crc32_incr_t crc = NULL;
    cgul_exception_t local = NULL;

    /* Allocate. */
    label = cgul_string__new(&local);
    crc = cgul_crc32_incr__new_appends_size(
              &local, 0x04c11db7, 0, 0xffffffff, 0, 0);
    if (local) {
        goto out;
    }

    /* Iterate over each file on the command line. */
    for (i = 1 ; i < argc ; ++i) {

        /* Add this file to the CRC. */
        cgul_crc32_incr__update_fname(&local, crc, argv[i]);
        value = cgul_crc32_incr__crc(&local, crc);
        if (local) {
            goto out;
        }

        /* Add this file to the list of files that have contributed to
         * the CRC. */
        if (cgul_string__get_length(&local, label) > 0) {
            cgul_string__append(&local, label, " + ");
        }
        cgul_string__append(&local, label, argv[i]);
        if (local) {
            goto out;
        }

        /* Print the CRC. */
        printf("%lu  %s\n", value, cgul_string__get_value(&local, label));
    }

 out:

    /* Catch exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (crc) {
        cgul_crc32_incr__delete(crc);
    }
    if (label) {
        cgul_string__delete(label);
    }

    return rv;
}
