#include <iostream>
#include <stdexcept>
#include "cgul_cxx.h"

using namespace std;

namespace {
    static const size_t BLOCK_SIZE = 16384;
}

int main(int argc, char* argv[])
{
    int rv = 0;
    int i = 0;
    unsigned long line_count = 0;
    unsigned long file_offset = 0;
    size_t rcount = 0;
    char* block = NULL;
    const char* line = NULL;
    FILE* f = NULL;
    cgul_exception_t local = NULL;
    cgul_crlf_cxx crlf;

    try {

        /* Verify the command line. */
        if (argc <= 1) {
            throw runtime_error(
                      "Usage: main_crlf <file.txt> [<file.txt> ... ]\n");
        }

        /* Allocate a block for reading. */
        block = (char*)cgul_libc__malloc(&local, BLOCK_SIZE);

#if 1
        /* Do not strip the UTF-8 byte-order mark. */
        crlf.set_strip_utf8_bom(0);
#endif

        /* Iterate over the command line arguments. */
        for (i = 1 ; i < argc ; ++i) {

            /* Open the file. */
            f = cgul_libc__fopen(&local, argv[i], "rb");
            if (local) {
                throw cgul_exception_cxx(local);
            }

            /* Reset the converter. */
            crlf.reset();

            /* Process the entire file. */
            for (;;) {

                /* Read a block from the file. */
                rcount = cgul_libc__fread(&local, block, 1, BLOCK_SIZE, f);
                if (local) {
                    throw cgul_exception_cxx(local);
                }

                /* EOF */
                if (rcount == 0) {
                    break;
                }

                /* Convert the block. */
                crlf.convert(block, rcount);
                if (local) {
                    throw cgul_exception_cxx(local);
                }

                /* Extract all of the lines in this block. */
                while ((line = crlf.get_line()) != NULL) {
                    line_count = crlf.get_line_count();
                    file_offset = crlf.get_line_offset();
                    printf("%5lu: %9lu: %s\n", line_count, file_offset, line);
                }
            }

            /* Handle the remainder if any. */
            if ((line = crlf.get_remainder()) != NULL) {
                line_count = crlf.get_line_count();
                file_offset = crlf.get_line_offset();
                printf("%5lu: %9lu: %s\n", line_count, file_offset, line);
            }

            /* Close the file. */
            fclose(f);
            f = NULL;
        }

    } catch (const exception& se) {
        cout << "*** Error: " << se.what() << endl;
    } catch (const cgul_exception_cxx& ce) {
        cout << "*** Error: " << ce.get_message() << endl;
    }

    if (f) {
        fclose(f);
    }

    if (block) {
        free(block);
    }

    return rv;
}
