#include "cgul.h"

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    int i = 0;
    const char* line = NULL;
    cgul_crlf_file_t f = NULL;
    cgul_exception_t local = NULL;

    /* Check usage. */
    if (argc == 1) {
        fprintf(stderr, "Usage: main_crlf_file <file.txt> [<file.txt> ...]\n");
        goto out;
    }

    /* Allocate. */
    f = cgul_crlf_file__new(&local);
    if (local) {
        goto out;
    }

    /* Iterate over the files passed in on the command line. */
    for (i = 1 ; i < argc ; ++i) {

        /* Open the file. */
        cgul_crlf_file__open_fname(&local,
                                   f,
                                   argv[i],
                                   1 /* is_block_buffered */);
        if (local) {
            goto out;
        }

#if 0
        /* Do not strip the leading UTF-8 byte-order mark. */
        cgul_crlf_file__set_strip_utf8_bom(&local, f, 0);
#endif

        /* Iterate over each line in f. */
        while ((line = cgul_crlf_file__get_line(&local, f)) != NULL) {
            unsigned long count = cgul_crlf_file__get_line_count(&local, f);
            unsigned long offset = cgul_crlf_file__get_line_offset(&local, f);
            cgul_libc__printf(&local, "%4lu: %5lu: %s\n", count, offset, line);
        }

        /* Close f. */
        cgul_crlf_file__close(&local, f);
    }

 out:

    /* Catch any exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    if (f) {
        cgul_crlf_file__delete(f);
    }

    return rv;
}
