/*
 * This program reads stdin into memory and then uses cgul_crlf_file
 * to parse the memory.
 */

#include "cgul.h"

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    const char* line = NULL;
    char* buf = NULL;
    size_t buf_size = 0;
    cgul_crlf_file_t crlf = NULL;
    cgul_exception_t local = NULL;

    /* Read stdin into buf. */
    cgul_libc__load_file(&local, stdin, &buf, &buf_size);
    if (local) {
        goto out;
    }

    /* Allocate. */
    crlf = cgul_crlf_file__new(&local);
    if (local) {
        goto out;
    }

    /* Open memory for crlf processing. */
    cgul_crlf_file__open_memory(&local, crlf, buf, buf_size);

    /* Iterate over each line. */
    while ((line = cgul_crlf_file__get_line(&local, crlf)) != NULL) {
        puts(line);
    }

 out:

    /* Handle exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (crlf) {
        cgul_crlf_file__delete(crlf);
    }
    if (buf) {
        free(buf);
    }

    return rv;
}
