/*
 * This program tests the cgul_csv_cxx::quote() static method which is
 * used to quote fields before writing them to a CSV file.  Also see
 * the main_csv_sort.c program for a much more extensive example of
 * this.
 */

#include <iostream>
#include "cgul_cxx.h"

using namespace std;

int
main(int argc,
     char* argv[])
{
    int rv = 0;

    try {

        cgul_string_cxx row;

        /*
         * Create a row for a CSV file by quoting each field and
         * manually inserting the commas between fields.
         */

        /* Simple strings. */
        cgul_csv_cxx::quote("1", row);
        row.append_char(',');
        cgul_csv_cxx::quote("2", row);
        row.append_char(',');
        cgul_csv_cxx::quote("3", row);
        row.append_char(',');

        /* String with embedded quote. */
        cgul_csv_cxx::quote("He said, \"What?\"", row);
        row.append_char(',');

        /* Multi-line text. */
        cgul_csv_cxx::quote("This\nis\nmulti-line\ntext.", row);

        /* Print the result. */
        cout << row.get_value() << endl;

    } catch (const cgul_exception_cxx& e) {
        cerr << "*** Error: " << e.get_message() << endl;
        rv = 1;
    }

    return rv;
}
