#include "cgul.h"

static int
print_value(cgul_exception_t* cex,
            void* value,
            void* data)
{
    int rv = 0;

    assert(cex);
    if (*cex) {
        goto out;
    }

    printf("%d\n", *(int*)value);
    rv = 1;

 out:
    return rv;
}

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    int i = 0;
    int* value = NULL;
    cgul_list_t l = NULL;
    cgul_exception_t local = NULL;

    /* Allocate. */
    l = cgul_list__new(&local);
    if (local) {
        goto out;
    }

    /* Populate the list. */
    for (i = 0 ; i < 5 ; ++i) {
        /* Allocate a new value to add to the list. */
        value = cgul_libc__calloc(&local, 1, sizeof(*value));
        if (local) {
            goto out;
        }
        /* Assign a value to the newly allocated memory. */
        *value = i;
        /* Add the value to the list. */
        cgul_list__push_back(&local, l, value);
        if (local) {
            goto out;
        }
        /* The list now owns value. */
        value = NULL;
    }

    /* Print the values on the list in forward and then reverse order. */
    cgul_list__foldl(&local, l, print_value, NULL);
    puts("=====");
    cgul_list__foldr(&local, l, print_value, NULL);

 out:

    /* Catch exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (l) {
        cgul_list__free_values(l);
        cgul_list__delete(l);
    }
    if (value) {
        free(value);
    }

    return rv;
}
