/**
 * This program generates 10M random unsigned long values and writes
 * them as text to standard output.
 */

#include <limits.h>
#include "cgul.h"

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    size_t i = 0;
    cgul_random_lcg256_t prng = NULL;
    cgul_big_integer_t one = NULL;
    cgul_big_integer_t max = NULL;
    cgul_big_integer_t next = NULL;
    cgul_exception_t local = NULL;

    /* Allocate. */
    prng = cgul_random_lcg256__new(&local);
    one = cgul_big_integer__new_from_long(&local, 1L);
    max = cgul_big_integer__new(&local);
    next = cgul_big_integer__new(&local);
    if (local) {
        goto out;
    }

    /* Set the _exclusive_ upper-bound of the range we are generating
     * to be one more than the largest value an "unsigned long" can
     * hold. */
    cgul_big_integer__assign_from_unsigned_long(&local, max, ULONG_MAX);
    cgul_big_integer__add(&local, max, max, one);
    if (local) {
        goto out;
    }

    /* Generate 10M random unsigned longs and write them to standard
     * output as text. */
    for (i = 0 ; i < 10000000 ; ++i) {
        cgul_random_lcg256__next_in_range(&local, prng, next, max);
        cgul_big_integer__print_as_hexadecimal(&local, next, stdout);
        puts("");
    }

 out:

    /* Catch exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (one) {
        cgul_big_integer__delete(one);
    }
    if (max) {
        cgul_big_integer__delete(max);
    }
    if (next) {
        cgul_big_integer__delete(next);
    }
    if (prng) {
        cgul_random_lcg256__delete(prng);
    }

    return rv;
}
