#include "cgul_cxx.h"

/* At the time of writing, the cgul_random_* classes do not have C++
 * wrappers which makes this test code have mixed C-style and
 * C++-style exceptions. */

struct point {
    int x;
    int y;
};

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    size_t i;
    size_t v_count = 10;
    struct point* v = NULL;
    cgul_random_lfg32_t prng = NULL;
    cgul_exception_t local = NULL;

    /* Allocate. */
    v = (struct point*)cgul_libc__calloc(&local, v_count, sizeof(*v));
    prng = cgul_random_lfg32__new(&local);
    if (local) {
        goto out;
    }

    /* Initialize. */
    for (i = 0 ; i < v_count ; ++i) {
        v[i].x = i;
        v[i].y = i + 1;
    }

    /* Shuffle. */
    try {
        cgul_shuffle_cxx::cgul_shuffle(v,
                                       v_count,
                                       sizeof(*v),
                                       (cgul_shuffle__next_in_range_t)
                                           cgul_random_lfg32__next_in_range,
                                       prng);
    } catch (const cgul_exception_cxx e) {
        local = cgul_exception__new("%s", e.get_message());
        goto out;
    }

#if 1
    /* Print the shuffled array. */
    for (i = 0 ; i < v_count ; ++i) {
        printf("(%6d, %6d)\n", v[i].x, v[i].y);
    }
#endif

 out:

    /* Handle exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (prng) {
        cgul_random_lfg32__delete(prng);
    }
    if (v) {
        free(v);
    }

    return rv;
}
