/*
 * For cgul_stream_reader, test automatic detection of UTF-8, UTF-16,
 * and UTF-32 streams.  This program reads all the streams and
 * converts them to UTF-8 for display so they should all be the same.
 */

#include "cgul_cxx.h"
#include <iostream>
#include <stdexcept>

using namespace std;

/**
 * Convert the Unicode stream <code>sr</code> to UTF-8 and print the
 * result.
 *
 * @param[in] sr <code>cgul_stream_reader_cxx</code> instance
 */
static void
main__print_stream(cgul_stream_reader_cxx& sr)
{
    cgul_wchar_t utf32 = CGUL_WCHAR__NUL;
    cgul_string_cxx s;

    /* Iterate over each character in the stream. */
    for (;;) {

        /* Read the next character. */
        if (!sr.read(&utf32)) {
            /* EOF */
            break;
        }

        /* Append the UTF-32 character to the UTF-8 accumulator. */
        s.append_wchar(utf32);
    }

    /* Print the result. */
    cout << s.get_value() << endl;
}

int
main(int argc,
     char* argv[])
{
    int rv = 0;

    try {

        cgul_string_cxx utf8;
        cgul_hstring_cxx utf16;
        cgul_wstring_cxx utf32;
        cgul_block_reader_memory_cxx br;
        cgul_stream_reader_cxx sr;

        /********************************************************************
         * UTF-8
         ********************************************************************/

        /* Create a UTF-8 stream with a Unicode Byte-Order Mark (BOM). */
        utf8.append_wchar((cgul_wchar_t)0xFEFF);
        utf8.append("Hello, World!");

        /* Open the stream. */
        br.open(utf8.get_value(), utf8.get_length());
        sr.reset();
        sr.add_block_reader(br);

        /* Print the stream. */
        main__print_stream(sr);

        /********************************************************************
         * UTF-16
         ********************************************************************/

        /* Create a UTF-16 stream with a Unicode Byte-Order Mark (BOM). */
        cgul_unicode_cxx::mbstohcs(utf8.get_value(),
                                   utf8.get_length(),
                                   /* skip_bom */ 0,
                                   utf16);

        /* Open the stream. */
        br.open(utf16.get_value(), 2 * utf16.get_length());
        sr.reset();
        sr.add_block_reader(br);

        /* Print the stream. */
        main__print_stream(sr);

        /********************************************************************
         * UTF-32
         ********************************************************************/

        /* Create a UTF-32 stream with a Unicode Byte-Order Mark (BOM). */
        cgul_unicode_cxx::mbstowcs(utf8.get_value(),
                                   utf8.get_length(),
                                   /* skip_bom */ 0,
                                   utf32);

        /* Open the stream. */
        br.open(utf32.get_value(), 4 * utf32.get_length());
        sr.reset();
        sr.add_block_reader(br);

        /* Print the stream. */
        main__print_stream(sr);

    } catch (const exception& e) {
        cerr << "*** Error: " << e.what() << endl;
        rv = 1;
    } catch (const cgul_exception_cxx& e) {
        cerr << "*** Error: " << e.get_message() << endl;
        rv = 1;
    }

    return rv;
}
