#include "cgul.h"

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    cgul_wchar_t utf32 = CGUL_WCHAR__NUL;
    cgul_string_t s = NULL;
    cgul_block_reader_file_t br = NULL;
    cgul_stream_reader_t istrm = NULL;
    cgul_exception_t local = NULL;

    /* Allocate. */
    s = cgul_string__new(&local);
    br = cgul_block_reader_file__new_from_fname(&local, "tower_of_babel.txt");
    istrm = cgul_stream_reader__new_from_block_reader(
        &local, cgul_block_reader_file__as_block_reader(&local, br));
    if (local) {
        goto out;
    }

    /* Iterate over each UTF-32 character in the input stream. */
    for (;;) {

        /* Get the next UTF-32 character. */
        if (!cgul_stream_reader__read(&local, istrm, &utf32)) {
            if (local) {
                goto out;
            }
            /* EOF */
            break;
        }

        /* Append the UTF-32 character to "s". */
        cgul_string__append_wchar(&local, s, utf32);
        if (local) {
            goto out;
        }
    }

    /* Print the result UTF-8 string to stdout. */
    printf("%s", cgul_string__get_value(&local, s));

 out:

    /* Handle exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (istrm) {
        cgul_stream_reader__delete(istrm);
    }
    if (br) {
        cgul_block_reader_file__delete(br);
    }
    if (s) {
        cgul_string__delete(s);
    }

    return rv;
}
