#include "cgul.h"

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    unsigned long i = 0;
    const char* ptr = NULL;
    //const char* s1 = ", World";
    //const char* s1 = "2206467108856483190734363163860902047173";
    //const char* s1 = "6467108856483190734363163860902047173953";
    const char* s1 = "761605204005594395539757990177246389314888757277006846643516817474408";
    cgul_string_t s2 = NULL;
    cgul_substring_t ss = NULL;
    cgul_big_integer_t seed = NULL;
    cgul_random_lcg256_t lcg = NULL;
    cgul_big_integer_t next = NULL;
    cgul_big_integer_t max = NULL;
    cgul_exception_t local = NULL;

    /* Allocate. */
    s2 = cgul_string__new(&local);
    ss = cgul_substring__new(&local);
    seed = cgul_big_integer__new_from_string(&local, "0x1234890172347134");
    lcg = cgul_random_lcg256__new_from_seed(&local, seed);
    next = cgul_big_integer__new(&local);
    max = cgul_big_integer__new_from_string(&local, "0x100000000000000000000000000000000");
    if (local) {
        goto out;
    }

#if 1
    /* Generate some text. */
    for (i = 0 ; i < 10000 ; ++i) {
        cgul_random_lcg256__next_in_range(&local, lcg, next, max);
        cgul_big_integer__append_as_decimal(&local, next, s2);
        if (local) {
            goto out;
        }
    }
#else
    /* Use stock "Hello, World!" as the text. */
    cgul_string__set_value(&local, s2, "Hello, World!");
    if (local) {
        goto out;
    }
#endif

#if 0
    printf("%s\n", cgul_string__get_value(&local, s2));
#endif

    /* Perform the search using the easy API. */
    ptr = cgul_substring__search_ez(&local,
                                    s1,
                                    strlen(s1),
                                    cgul_string__get_value(&local, s2),
                                    cgul_string__get_length(&local, s2));
    if (local) {
        goto out;
    }

    /* Print the index. */
    if (ptr) {
        printf("(ez) index = %lu\n",
               (unsigned long)(ptr - cgul_string__get_value(&local, s2)));
    } else {
        printf("No match.\n");
    }

    /* Perform the same search using the regular API. */
    cgul_substring__set_value(&local, ss, s1, strlen(s1));
    ptr = cgul_substring__search(&local,
                                 ss,
                                 cgul_string__get_value(&local, s2),
                                 cgul_string__get_length(&local, s2));
    if (local) {
        goto out;
    }

    /* Print the index. */
    if (ptr) {
        printf("     index = %lu\n",
               (unsigned long)(ptr - cgul_string__get_value(&local, s2)));
    } else {
        printf("No match.\n");
    }

 out:

    /* Catch exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (max) {
        cgul_big_integer__delete(max);
    }
    if (next) {
        cgul_big_integer__delete(next);
    }
    if (lcg) {
        cgul_random_lcg256__delete(lcg);
    }
    if (seed) {
        cgul_big_integer__delete(seed);
    }
    if (ss) {
        cgul_substring__delete(ss);
    }
    if (s2) {
        cgul_string__delete(s2);
    }

    return rv;
}
