/*
 * This program intentionally does not call setlocale().  This means
 * the LANG environment variable will not alter the way this program
 * runs.  This is a good thing when you need to disable all
 * translations when reading and writing from file.  For example, if
 * you need to convert a file to UTF-8 and the user has the LANG
 * environment variable set to zh_TW.big5, calling setlocale() is
 * going to cause the output to be translated to BIG-5.
 *
 * Because we already know that the Unicode Byte-Order Mark (BOM) is
 * 0xef, 0xbb, and 0xbf when converted to UTF-8, we could just print
 * it directly using the following:
 *
 *     printf("%c%c%c", 0xef, 0xbb, 0xbf);
 *
 * The point to this program however it show how to manually convert
 * between UTF-32 and UTF-8.
 */

#include "cgul.h"

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    cgul_string_t utf8 = NULL;
    cgul_wstring_t utf32 = NULL;
    cgul_exception_t local = NULL;

    /* Allocate. */
    utf8 = cgul_string__new(&local);
    utf32 = cgul_wstring__new(&local);
    if (local) {
        goto out;
    }

    /* Append the Unicode Byte-Order Mark (BOM) to the UTF-32 string. */
    cgul_wstring__append_char(&local, utf32, CGUL_WCHAR__BOM);
    if (local) {
        goto out;
    }

    /* Convert the UTF-32 string to UTF-8. */
    cgul_unicode__wcstombs(&local,
                           cgul_wstring__get_value(&local, utf32),
                           cgul_wstring__get_length(&local, utf32),
                           0, /* skip_bom */
                           utf8);
    if (local) {
        goto out;
    }

    /* Print the UTF-8 string. */
    printf("%s", cgul_string__get_value(&local, utf8));

 out:

    /* Catch exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (utf8) {
        cgul_string__delete(utf8);
    }
    if (utf32) {
        cgul_wstring__delete(utf32);
    }

    return rv;
}
