/**
 * This program reads a UTF-8 text file and for each Unicode character
 * prints the character followed by its code point in hexadecimal and
 * decimal.
 *
 * @file
 * @brief dump contents of UTF-8 file
 */

#include "cgul.h"

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    size_t i = 0;
    size_t length = 0;
    const char* utf8 = NULL;
    cgul_wchar_t* utf32 = NULL;
    cgul_wstring_t ws = NULL;
    cgul_crlf_file_t crlf = NULL;
    cgul_exception_t local = NULL;
    char buf[CGUL_WCHAR__MB_LEN_MAX + 1];

    /* Allocate ws. */
    ws = cgul_wstring__new(&local);
    if (local) {
        goto out;
    }

    /* Allocate crlf. */
    crlf = cgul_crlf_file__new_from_file(&local,
                                         stdin,
                                         1 /* is_block_buffered */);
    if (local) {
        goto out;
    }

    /* crlf should not strip leading UTF-8 BOM. */
    cgul_crlf_file__set_strip_utf8_bom(&local, crlf, 0);

    /* Iterate over each line of standard input. */
    while ((utf8 = cgul_crlf_file__get_line(&local, crlf)) != NULL) {

        /* Convert from UTF-8 to UTF-32. */
        cgul_wstring__clear(ws);
        cgul_unicode__mbstowcs(&local, utf8, strlen(utf8), 0, ws);
        if (local) {
            goto out;
        }

        /* Unpack "ws". */
        utf32 = cgul_wstring__get_value(&local, ws);
        length = cgul_wstring__get_length(&local, ws);

        /* Print results for this line. */
        for (i = 0 ; i < length ; ++i) {
            /* Convert back to UTF-8 for display. */
            cgul_unicode__wctomb(&local, utf32[i], buf);
            if (local) {
                goto out;
            }
            /* Display. */
            printf("%s  -->  %#8lx  %8ld\n",
                   buf,
                   (long unsigned)utf32[i],
                   (long unsigned)utf32[i]);
        }
    }

 out:

    /* Catch exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (crlf) {
        cgul_crlf_file__delete(crlf);
    }
    if (ws) {
        cgul_wstring__delete(ws);
    }

    return rv;
}
