/**
 * This program reads a UTF-8 text file and for each Unicode character
 * prints the character followed by its code point in hexadecimal and
 * decimal.
 *
 * @file
 * @brief dump contents of UTF-8 file
 */

#include <stdexcept>
#include "cgul_cxx.h"

using namespace std;

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    size_t i = 0;
    size_t length = 0;
    const char* utf8 = NULL;
    cgul_wchar_t* utf32 = NULL;
    cgul_wstring_cxx ws;
    char buf[CGUL_WCHAR__MB_LEN_MAX + 1];

    try {

        /* Construct line reader. */
        cgul_crlf_file_cxx crlf(stdin);

        /* crlf should not strip leading UTF-8 BOM. */
        crlf.set_strip_utf8_bom(0);

        /* Iterate over each line of standard input. */
        while ((utf8 = crlf.get_line()) != NULL) {

            /* Convert from UTF-8 to UTF-32. */
            ws.clear();
            cgul_unicode_cxx::mbstowcs(utf8, strlen(utf8), 0, ws);

            /* Unpack "ws". */
            utf32 = ws.get_value();
            length = ws.get_length();

            /* Print results for this line. */
            for (i = 0 ; i < length ; ++i) {
                /* Convert back to UTF-8 for display. */
                cgul_unicode_cxx::wctomb(utf32[i], buf);
                /* Display. */
                printf("%s  -->  %#8lx  %8ld\n",
                       buf,
                       (long unsigned)utf32[i],
                       (long unsigned)utf32[i]);
            }
        }

    } catch (const exception& e) {
        fprintf(stderr, "*** Error: %s\n", e.what());
        rv = 1;
    } catch (const cgul_exception_cxx& e) {
        fprintf(stderr, "*** Error: %s\n", e.get_message());
        rv = 1;
    }

    return rv;
}
