#include <iostream>
#include <stdexcept>
#include "cgul_cxx.h"

using namespace std;

/**
 * Append the <code>unsigned long</code> values on the half-open range
 * <code>[start, stop)</code> to the vector <code>v</code>.  The
 * client is responsible for calling <code>free()</code> on all the
 * pointers added to <code>v</code>.  If an error occurs, an exception
 * is thrown.
 *
 * @param[in] v vector
 * @param[in] start inclusive starting value
 * @param[in] stop exclusive stopping value
 */
static void
append_values(cgul_vector_cxx& v,
              unsigned long int start,
              unsigned long int stop)
{
    unsigned long int i = 0;
    unsigned long int* n = NULL;

    /* Iterate over the values */
    for (i = start ; i < stop ; ++i) {

        /* Allocate space for the value. */ 
        n = (long unsigned int*)cgul_libc_cxx::calloc(1, sizeof(*n));

        /* Set the value. */
        *n = i;

        /* Append the value to the vector. */
        v.push_back(n);
    }
}

/**
 * Print the vector <code>v</code> to the output file
 * <code>fout</code> where each entry is output with the prefix
 * <code>prefix[<index>]</code>.
 *
 * @param[in] v vector
 * @param[in] prefix prefix
 * @param[in] ostrm output stream
 */
static void
print_vector(cgul_vector_cxx& v,
             const char* prefix,
             ostream& ostrm)
{
    unsigned long int i = 0;
    unsigned long int i_max = 0;
    unsigned long int* n = NULL;

    /* Print the vector. */
    i_max = v.get_size();
    for (i = 0 ; i < i_max ; ++i) {
        n = (unsigned long*)v.at(i);
        ostrm << prefix << "[" << i << "] = " << *n << endl;
    }
}

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    cgul_vector_cxx v1;
    cgul_vector_cxx v2;

    try {

        /* Add [0 - 5) to the v1 and [5 - 10) to v2. */
        append_values(v1, 0, 5);
        append_values(v2, 5, 10);

        /* Print v1 and v2. */
        print_vector(v1, "v1", cout);
        print_vector(v2, "v2", cout);

        cout << "-----\n";

        /* Swap the vectors. */
        v1.swap(v2);

        /* Print v1 and v2. */
        print_vector(v1, "v1", cout);
        print_vector(v2, "v2", cout);

    } catch (const exception& e) {
        cerr << "*** Error: " << e.what() << endl;
        rv = 1;
    }

    /* Clean up. */
    v2.free_values();
    v1.free_values();

    return rv;
}
